/* 6545 CRTC Header */

#ifndef HEADER_CRTC_H
#define HEADER_CRTC_H

#include "z80.h"

#define CRTC_HTOT           0
#define CRTC_HDISP          1
#define CRTC_HSYNC_POS      2
#define CRTC_SYNC_WIDTH     3
#define CRTC_VTOT           4
#define CRTC_VTOT_ADJ       5
#define CRTC_VDISP          6
#define CRTC_VSYNC_POS      7
#define CRTC_MODE           8
#define CRTC_SCANLINES      9
#define CRTC_CUR_START      10
#define CRTC_CUR_END        11
#define CRTC_DISP_START_H   12
#define CRTC_DISP_START_L   13
#define CRTC_CUR_POS_H      14
#define CRTC_CUR_POS_L      15
#define CRTC_LPEN_H         16
#define CRTC_LPEN_L         17
#define CRTC_SETADDR_H      18
#define CRTC_SETADDR_L      19

#define CRTC_DOSETADDR      31

// default video resolution
#define VDUWIDTH  640
#define VDUHEIGHT 275
#define VDUPOSX 0
#define VDUPOSY 0

// CGA Colour monitor intensity values
#define C_FI    252     // colour full intensity
#define C_HI    168     // colour half intensity
#define C_LI    84      // colour low intensity

// EGA Colour monitor intensity values
#define E_FF    255     // colour full intensity
#define E_AA    170     // colour half intensity
#define E_55    85      // colour low intensity

// Green monitor
#define MONGR_BGR   0
#define MONGR_BGG   0
#define MONGR_BGB   0
#define MONGR_BGR_I 0
#define MONGR_BGG_I 80
#define MONGR_BGB_I 0
#define MONGR_FGR   0
#define MONGR_FGG   210
#define MONGR_FGB   0
#define MONGR_FGR_I 0
#define MONGR_FGG_I 252
#define MONGR_FGB_I 0

// Amber monitor
#define MONAM_BGR   0
#define MONAM_BGG   0
#define MONAM_BGB   0
#define MONAM_BGR_I 158
#define MONAM_BGG_I 110
#define MONAM_BGB_I 0
#define MONAM_FGR   238
#define MONAM_FGG   166
#define MONAM_FGB   0
#define MONAM_FGR_I 238
#define MONAM_FGG_I 200
#define MONAM_FGB_I 0

// White monitor
#define MONWB_BGR   0
#define MONWB_BGG   0
#define MONWB_BGB   0
#define MONWB_BGR_I 128
#define MONWB_BGG_I 128
#define MONWB_BGB_I 128
#define MONWB_FGR   192
#define MONWB_FGG   192
#define MONWB_FGB   192
#define MONWB_FGR_I 255
#define MONWB_FGG_I 255
#define MONWB_FGB_I 255

// Black monitor
#define MONBW_BGR   255
#define MONBW_BGG   255
#define MONBW_BGB   255
#define MONBW_BGR_I 84
#define MONBW_BGG_I 84
#define MONBW_BGB_I 84
#define MONBW_FGR   0
#define MONBW_FGG   0
#define MONBW_FGB   0
#define MONBW_FGR_I 115
#define MONBW_FGG_I 115
#define MONBW_FGB_I 115

// User monitor (Yellow on Blue)
#define MONUSR_BGR   0
#define MONUSR_BGG   0
#define MONUSR_BGB   168
#define MONUSR_BGR_I 84
#define MONUSR_BGG_I 84
#define MONUSR_BGB_I 252
#define MONUSR_FGR   252
#define MONUSR_FGG   252
#define MONUSR_FGB   84
#define MONUSR_FGR_I 252
#define MONUSR_FGG_I 252
#define MONUSR_FGB_I 150

extern int disp_start;

void crtc_setmonitor (void);

int crtc_init (void);
int crtc_deinit (void);
int crtc_reset (void);

void crtc_redraw (void);
void crtc_set_redraw (void);
void crtc_redraw_char (int addr, int dostdout);

void crtc_lpen (int addr);

int crtc_vblank (void);
uint16_t crtc_status (uint16_t port, struct z80_port_read *port_struct);
void crtc_address (uint16_t port, uint8_t data, struct z80_port_write *port_struct);
uint16_t crtc_data_r (uint16_t port, struct z80_port_read *port_struct);
void crtc_data_w (uint16_t port, uint8_t data, struct z80_port_write *port_struct);

void crtc_update (void);
void crtc_regdump (void);
void crtc_set_mon_table (int pos, int col);
void crtc_configure (int aspect);
int crtc_set_flash_rate (int n);
void crtc_clock (int cpuclock);

typedef struct crtc_t
   {
    int video;
    int vblank_method;
    int monitor;
    int flashvideo;
    int aspect;
    int yscale;
    int extendram;
    int latchrom;
    int colour_cont;
    int hdisp;
    int vdisp;
    int disp_start;
    int scans_per_row;
    int std_col_type;
    int flashbits;
    int resized;
    int flashrate;
    int flashvalue_c;
    int flashvalue_t;
    int update;
   }crtc_t;

#endif     /* HEADER_CRTC_H */
