/* VDU Header */

#ifndef HEADER_VDU_H
#define HEADER_VDU_H

#include "ubee512.h"
#include "z80.h"

// Alpha+ (Premium) variables, 8K for Screen, Colour and attribute, 32K for PCG
#define SCR_RAM_BANKS 4
#define COL_RAM_BANKS 4
#define ATT_RAM_BANKS 4
#define PCG_RAM_BANKS 17

#define CHR_ROM_SIZE 0x4000
#define SCR_RAM_SIZE 0x0800 * SCR_RAM_BANKS
#define COL_RAM_SIZE 0x0800 * COL_RAM_BANKS
#define ATT_RAM_SIZE 0x0800 * ATT_RAM_BANKS
#define PCG_RAM_SIZE 0x0800 * PCG_RAM_BANKS

int vdu_init (void);
int vdu_deinit (void);
int vdu_reset (void);

void vdu_switchvideo_in (void);
void vdu_switchvideo_out (void);

uint8_t vdu_vidmem_r (uint32_t addr, struct z80_memory_read_byte *mem_s);
void vdu_vidmem_w (uint32_t addr, uint8_t data, struct z80_memory_write_byte *mem_s);

uint16_t vdu_colcont_r (uint16_t port, struct z80_port_read *port_s);
void vdu_colcont_w (uint16_t port, uint8_t data, struct z80_port_write *port_s);

uint16_t vdu_colwait_r (uint16_t port, struct z80_port_read *port_s);
void vdu_colwait_w (uint16_t port, uint8_t data, struct z80_port_write *port_s);

uint16_t vdu_lvdat_r (uint16_t port, struct z80_port_read *port_s);
void vdu_lvdat_w (uint16_t port, uint8_t data, struct z80_port_write *port_s);

void vdu_latchrom (uint16_t port, uint8_t data, struct z80_port_write *port_s);

typedef struct vdu_t
   {
    char chr_rom[CHR_ROM_SIZE];
    char scr_ram[SCR_RAM_SIZE];
    char col_ram[COL_RAM_SIZE];
    char att_ram[ATT_RAM_SIZE];
    char pcg_ram[PCG_RAM_SIZE];  // last bank is a dummy bank
   }vdu_t;

#endif /* HEADER_VDU_H */
