/* VIDEO Header */

#ifndef HEADER_VIDEO_H
#define HEADER_VIDEO_H

#ifdef USE_OPENGL
#include <gl.h>
#endif

// default video resolution
#define VIDEO_WIDTH 640
#define VIDEO_HEIGHT 275

// video types
#define VIDEO_SDLSW 0
#define VIDEO_SDLHW 1
#define VIDEO_GL 2

// video depths
#define VIDEO_8 0
#define VIDEO_8GS 1
#define VIDEO_16 2
#define VIDEO_32 3

#ifdef USE_OPENGL
// OpenGL video filter values
#define VIDEO_SOFT 0
#define VIDEO_SHARP 1

// OpenGL screen size incrementing
#define VIDEO_INCREMENT_PERCENT 2
#define VIDEO_MIN_PERCENT 5
#define VIDEO_MAX_PERCENT 99 // don't use 100%

// OpenGL aspect ratios as decimal value
#define VIDEO_ASPECT_BEE 1.33

// Full screen aspect handling
#define VIDEO_ASPECT_FS_AUTO    0
#define VIDEO_ASPECT_FS_STRETCH 1
#define VIDEO_ASPECT_FS_KEEP    2

// OpenGL vsync values
#define VIDEO_VSYNC_OFF 0
#define VIDEO_VSYNC_ON 1
#endif

// Convert floating point display ratios to an integer for testing
#define VIDEO_DISP_RATIO (int)(disp_ratio * 100)
#define VIDEO_UBEE_RATIO (int)(video.aspect_bee * 100)
#define VIDEO_STRETCH_RATIO (int)(video.stretch_fs * 100)

int video_init (void);
int video_deinit (void);
int video_reset (void);
void video_convert_mouse_to_crtc_xy (int mouse_x, int mouse_y, int *x, int *y);
void video_convert_crtc_to_mouse_xy (int crtc_x, int crtc_y, int *x, int *y);
int video_create_surface (int crt_w, int crt_h, int win_w, int win_h);
void video_renderer (void);

#ifdef USE_OPENGL
void video_gl_size_set (int p);
void video_gl_size_increment (int dir);
void video_gl_resize_event (void);
void video_gl_filter_toggle (void);
int video_gl_aspect_bee (float aspect);
int video_gl_aspect_mon (float aspect);
int video_gl_stretch_fs (float aspect);
int video_gl_size_pixels (int pixels);
int video_gl_size_percent (int percent);
void video_gl_filter_update (void);
#endif

typedef void (*video_putpixel_fast_fn)(int x, int y, int val);
void video_putpixel (int x, int y, int col);
static inline void video_putpixel_fast(int x, int y, int val)
{
 extern video_putpixel_fast_fn video_putpixel_fast_p;
 (*video_putpixel_fast_p)(x, y, val);
}
int video_toggledisplay (void);
void video_update (void);
void video_command (int cmd, int p);

#ifdef USE_OPENGL
typedef struct video_gl_t
   {
    GLuint texture;

    GLsizei texture_w;
    GLsizei texture_h;
    GLfloat tex_coord[4];

    GLenum target;
    GLenum format;
    GLenum type;
    GLint filter;

    void* pixels;
    int pitch;
   }video_gl_t;
#endif

typedef struct video_t
   {
    int desktop_w;
    int desktop_h;

    int offset_x;
    int offset_y;

    int depth;
    int type;
    int fullscreen;

    int flags;
    int bpp;

#ifdef USE_OPENGL
    int gl_basedim_w;
    int gl_basedim_h;
    int gl_imagedim_w;
    int gl_imagedim_h;

    int maximised_w;
    int maximised_h;

    int last_win_w;
    int last_win_h;

    int maximised;

    int filter_fs;
    int filter_win;
    int filter_max;

    float aspect_mon;
    float aspect_bee;
    float stretch_fs;
    int aspect_fs;

    int max;
    int vsync;

    int initial_x_pixels;
    int initial_x_percent;
    int percent_size;
#endif
   }video_t;

#endif /* HEADER_VIDEO_H */
